declare module 'viewar-api' {
  namespace viewarApi {
    function init(apiConfig?: ApiConfig, window?: any): any;

    // Template for key/value pairs (JSON).
    interface StringTMap<T> {
      [key: string]: T;
    }

    //-------------
    // Types
    //-------------

    export type ApiConfig = {
      waitForDebugger?: boolean;
      mockProxy?: string;
      visualizeTouches?: boolean;
      logToScreen?: boolean;
      logToServer?: boolean;
    };

    export type RoomDescription = {};

    export type SurfaceQuery = {
      type: string;
      parameters: object;
    };

    export type AreaResult = {
      areas: Array<number>;
    };

    export type EmailObject = {
      subject: string;
      message: string;
      recipients: Array<string>;
      cc: Array<string>;
      bcc: Array<string>;
      attachments: Array<string>;
    };

    export type Interaction = {
      manipulationPlane: string;
      translation: boolean;
      rotation: boolean;
      scaling: boolean;
    };

    export type Vector3 = {
      x: number;
      y: number;
      z: number;
    };

    export type Quaternion = {
      w: number;
      x: number;
      y: number;
      z: number;
    };

    export type MetaData = {};

    export type Pose = {
      position?: Vector3;
      orientation?: Quaternion;
      scale?: Vector3;
    };

    export type ContainerInsertionParams = {
      parent: Container;
      pose: Pose;
      interaction: Interaction;
      visible: boolean;
      type: string;
    };

    export type ModelInsertionParams = {
      id?: string,
      parent?: Container;
      pose?: Pose;
      interaction?: Interaction;
      visible?: boolean;
      propertyValues?: StringTMap<string>;
    };

    export type BulkInsertionInfo = {
      model: Model | string;
      instanceProps?: ModelInsertionParams;
      object?: insertionParams;
    };

    export type FeaturePointHit = {
      intersection: Vector3;
      distance: number;
      position: Vector3;
      offset: number;
    };

    export type InstanceHitIntersection = {
      faceID: number;
      x: number;
      y: number;
      z: number;
      normalX: number;
      normalY: number;
      normalZ: number;
    };

    export type InstanceHit = {
      instanceId: string;
      mesh: string;
      meshid: string;
      modelid: string;
      submesh: string;
      submeshid: string;
      surfaceId: string;
      intersection: InstanceHitIntersection;
    };

    export type Ray = {
      origin: Vector3;
      direction: Vector3;
    };

    export type RayHits = {
      featurePoints: Array<FeaturePointHit>;
      floors: Array<Vector3>;
      instances: Array<InstanceHit>;
      ray: Ray;
    };

    export type SceneExportResult = {
      success: boolean;
      error: string;
      path: string;
    };

    export type VersionInfo = {
      app: string;
      api: string;
      core: string;
    };

    export type AppConfig = {
      appId: string;
      channelId: string;
      deviceCountry: string;
      deviceLanguage: string;
      deviceModel: string;
      deviceName: string;
      deviceOS: string;
      deviceRam: number;
      deviceType: string;
      fakedBundle: boolean;
      freezeFrameInfo: Array<object>;
      host: string;
      modelTree: object;
      modes: Array<string>;
      pkId: string;
      stageList: Array<string>;
      storageKey: string;
      trackerList: Array<object>;
      uiConfig: object;
      version: VersionInfo;
      visualizeTouches: boolean;
    };

    export type ColorRgb = {
      r: number;
      g: number;
      b: number;
    };

    export type ControllableCameraPose = {
      position: Vector3;
      orientation: Quaternion;
    };

    export type PerspectiveCameraPose = {
      position: Vector3;
      lookAt: Vector3;
    };

    export type ProjectSpecification = {
      name: string;
      screenshotUrl: string;
      info: object;
      data: object;
    };

    export type SyncParams = {
      pollInterval?: number;
      send?: boolean;
      receive?: boolean;
    };

    export type ARCoreOptions = {
      autoConfirmGroundPosition: boolean;
    };

    export type MediaPipeOptions = {
      autoConfirmGroundPosition: boolean;
    };

    export type ARKitOptions = {
      autoConfirmGroundPosition: boolean;
    };

    export type QuuppaOptions = {
      autoConfirmGroundPosition: boolean;
    };

    export type VisualixOptions = {
      autoConfirmGroundPosition: boolean;
    };    

    export type InfsoftOptions = {
      autoConfirmGroundPosition: boolean;
    };

    export type GPSOptions = {
      autoConfirmGroundPosition: boolean;
    };

    export type SixDegreesOptions = {
      autoConfirmGroundPosition: boolean;
    };

    export type IndoorAtlasOptions = {
      autoConfirmGroundPosition: boolean;
    };
    
    export type NavVisOptions = {
      autoConfirmGroundPosition: boolean;
    };

    export type ScanVolume = {
      size?: Vector3;
      resolution?: Vector3;
      offset?: Vector3;
    };

    export type SensorStatus = {
      connected: boolean;
      batteryLow: boolean;
      movingTooFast: boolean;
    };

    //-------------
    // Interfaces
    //-------------
    // Getters as readonly members.

    interface Emitter {
      on(eventName: string, handler: Function): void;
      off(eventName: string, handler: Function): void;
      once(eventName: string, handler: Function): void;
    }

    interface Mesh {
      readonly id: string;
      readonly name: string;
      readonly vertexCount: number;
      readonly material: string;
      readonly visibility: boolean;
      readonly collision: boolean;
      readonly wireframeMode: string;
      setMeshMaterial(material: string): Promise<undefined>;
      setMeshVisibility(visibility: boolean): Promise<undefined>;
      setMeshCollision(collision: boolean): Promise<undefined>;
      setMeshWireframeMode(wireFrameMode: string): Promise<undefined>;
    }

    interface TrackingTarget extends Emitter {
      readonly name: string;
      readonly type: string;
      readonly tracked: boolean;
      readonly container: Container;
    }

    interface MeshScanTrackingProvider {
      readonly scanning: boolean;
      startMeshScan(): Promise<undefined>;
      stopMeshScan(): Promise<undefined>;
      resetMeshScan(): Promise<undefined>;
      getMeshList(): Promise<Array<Mesh>>;
    }

    interface FloorOffsetTrackingProvider {
      readonly floorOffset: number;
      setFloorOffset(height: number): Promise<undefined>;
      getFloorOffset(): Promise<number>;
    }

    interface GroundConfirmTrackingProvider {
      readonly groundConfirmed: boolean;
      confirmGroundPosition(): Promise<undefined>;
    }

    interface TargetTrackingProvider {
      readonly targets: Array<TrackingTarget>;
      setTrackingTargets(newTargets: Array<object>): Promise<undefined>;
    }

    interface GenericTargetTrackingProvider
      extends TargetTrackingProvider,
        GroundConfirmTrackingProvider,
        FloorOffsetTrackingProvider,
        Tracker {}

    interface BasicTrackingProvider {
      readonly tracking: boolean;
    }

    interface Tracker extends Emitter {
      readonly active: boolean;
      readonly name: string;
      readonly tracking: boolean;
      activate(): Promise<boolean>;
      deactivate(): Promise<boolean>;
      reset(): Promise<undefined>;
    }

    interface ARCoreTracker
      extends TargetTrackingProvider,
        GroundConfirmTrackingProvider,
        Tracker {
      readonly options: ARCoreOptions;
      setOptions(newOptions: ARCoreOptions): void;
    }

    interface MediaPipeTracker
      extends TargetTrackingProvider,
        GroundConfirmTrackingProvider,
        Tracker {
      readonly options: MediaPipeOptions;
      setOptions(newOptions: MediaPipeOptions): void;
    }

    interface ARKitTracker
      extends TargetTrackingProvider,
        GroundConfirmTrackingProvider,
        Tracker {
      readonly options: ARKitOptions;
      readonly scanning: boolean;
      readonly scanVolume: ScanVolume;
      readonly sensorStatus: SensorStatus;
      setOptions(newOptions: ARKitOptions): void;
      resetMeshScan(): Promise<undefined>;
      startMeshScan(): Promise<undefined>;
      stopMeshScan(): Promise<undefined>;
      getMeshList(): Promise<Array<Mesh>>;
      fillMeshHoles(): Promise<undefined>;
      colorizeMesh(): Promise<undefined>;
      setScanVolume(scanVolume: ScanVolume, resetTracking?: boolean);
      getStatus(): Promise<object>;
      getTrackingQuality(): Promise<string>;
    }

    interface QuuppaTracker
      extends TargetTrackingProvider,
        GroundConfirmTrackingProvider,
        Tracker {
      readonly options: QuuppaOptions;
      
      setOptions(newOptions: QuuppaOptions): void;
    }

    interface VisualixTracker
      extends TargetTrackingProvider,
        GroundConfirmTrackingProvider,
        Tracker {
      readonly options: VisualixOptions;
      setOptions(newOptions: VisualixOptions): void;
    }

    interface InfsoftTracker
      extends TargetTrackingProvider,
        GroundConfirmTrackingProvider,
        Tracker {
      readonly options: InfsoftOptions;
      setOptions(newOptions: InfsoftOptions): void;
    }

    interface GPSTracker
      extends TargetTrackingProvider,
        GroundConfirmTrackingProvider,
        Tracker {
      readonly options: GPSOptions;
      setOptions(newOptions: GPSOptions): void;
    }

    interface PlacenoteTracker
      extends TargetTrackingProvider,
        GroundConfirmTrackingProvider,
        Tracker {
      readonly activeTrackingMap: string;
      readonly trackingMapId: string;
      generateTrackingMapId(): Promise<string>;
      saveTrackingMap(mapId?: string): Promise<string>;
      loadTrackingMap(mapId: string): Promise<boolean>;
      deleteTrackingMap(mapId: string): Promise<undefined>;
    }

    interface StructureTracker
      extends GroundConfirmTrackingProvider,
        BasicTrackingProvider,
        Tracker {
      readonly scanning: boolean;
      readonly scanVolume: ScanVolume;
      readonly sensorStatus: SensorStatus;
      resetMeshScan(): Promise<undefined>;
      startMeshScan(): Promise<undefined>;
      stopMeshScan(): Promise<undefined>;
      getMeshList(): Promise<Array<Mesh>>;
      fillMeshHoles(): Promise<undefined>;
      colorizeMesh(): Promise<undefined>;
      setScanVolume(scanVolume: ScanVolume, resetTracking?: boolean);
      getStatus(): Promise<object>;
      getTrackingQuality(): Promise<string>;
    }

    interface StructureBridgeTracker
      extends GroundConfirmTrackingProvider,
        BasicTrackingProvider,
        Tracker {
      readonly scanning: boolean;
      readonly scanVolume: ScanVolume;
      readonly sensorStatus: SensorStatus;
      resetMeshScan(): Promise<undefined>;
      startMeshScan(): Promise<undefined>;
      stopMeshScan(): Promise<undefined>;
      getMeshList(): Promise<Array<Mesh>>;
      fillMeshHoles(): Promise<undefined>;
      colorizeMesh(): Promise<undefined>;
      setScanVolume(scanVolume: ScanVolume, resetTracking?: boolean);
      getStatus(): Promise<object>;
      getTrackingQuality(): Promise<string>;
    }

    interface EighthWallTracker
      extends GroundConfirmTrackingProvider,
        BasicTrackingProvider,
        Tracker {
          readonly options: ARKitOptions;
      readonly scanning: boolean;
      readonly scanVolume: ScanVolume;
      readonly sensorStatus: SensorStatus;
      setOptions(newOptions: ARKitOptions): void;
      resetMeshScan(): Promise<undefined>;
      startMeshScan(): Promise<undefined>;
      stopMeshScan(): Promise<undefined>;
      getMeshList(): Promise<Array<Mesh>>;
      fillMeshHoles(): Promise<undefined>;
      colorizeMesh(): Promise<undefined>;
      setScanVolume(scanVolume: ScanVolume, resetTracking?: boolean);
      getStatus(): Promise<object>;
      getTrackingQuality(): Promise<string>;
    }

    interface VuforiaTracker
      extends TargetTrackingProvider,
        GroundConfirmTrackingProvider,
        FloorOffsetTrackingProvider,
        Tracker {
      readonly canConfirmGround: boolean;
      captureUserTarget(size: number): Promise<undefined>;
      removeUserTarget(): Promise<undefined>;
    }

    interface WikitudeTracker
      extends TargetTrackingProvider,
        GroundConfirmTrackingProvider,
        FloorOffsetTrackingProvider,
        Tracker {}

    interface SixDegreesTracker
      extends MeshScanTrackingProvider,
        GroundConfirmTrackingProvider,
        BasicTrackingProvider,
        Tracker {
      readonly options: SixDegreesOptions;
      setOptions(newOptions: SixDegreesOptions): void;
    }

    interface IndoorAtlasTracker
      extends TargetTrackingProvider,
        GroundConfirmTrackingProvider,
        Tracker {
      readonly options: IndoorAtlasOptions;
      setOptions(newOptions: IndoorAtlasOptions): void;
    }

    interface NavVisTracker
      extends TargetTrackingProvider,
        GroundConfirmTrackingProvider,
        Tracker {
      readonly options: NavVisOptions;
      setOptions(newOptions: NavVisOptions): void;
    }

    interface Trackers extends StringTMap<Tracker> {
      ARCore: ARCoreTracker;
      ARKit: ARKitTracker;
      Structure: StructureTracker;
      StructureBridge: StructureBridgeTracker;
      EighthWall: EighthWallTracker;
      Vuforia: VuforiaTracker;
      Wikitude: WikitudeTracker;
      Placenote: PlacenoteTracker;
      VisionLib: GenericTargetTrackingProvider;
      Remote: GenericTargetTrackingProvider;
      SixDegrees: SixDegreesTracker;
    }

    interface SyncManager {
      readonly syncId: string;
      register(
        propertyName: string,
        getter?: Function,
        setter?: Function
      ): void;
      unregister(propertyName: string): void;
      startLiveSync(sessionId: string, params: SyncParams): Promise<undefined>;
      stopLiveSync(): Promise<undefined>;
    }

    interface Storage {
      read(name: string): Promise<string>;
      write(name: string, content: string, format?: string): Promise<undefined>;
      remove(name: string): Promise<undefined>;
    }

    interface LocalStorage extends Storage {}

    interface CloudStorage extends Storage {
      storageKey: string;
    }

    interface Storages {
      readonly local: LocalStorage;
      readonly cloud: CloudStorage;
    }

    interface ScreenshotManager {
      takeScreenshot(): Promise<undefined>;
      saveScreenshot(subFolder: string): Promise<string>;
      deleteScreenshot(screenshotPath: string, subFolder: string);
      emailScreenshot(): Promise<undefined>;
      shareScreenshot(
        socialMediaService: string,
        screenshotPath: string
      ): Promise<undefined>;
      saveScreenshotToGallery(): Promise<undefined>;
    }

    interface Project {
      name: string;
      info: object;
      customData: any;
      readonly id: string;
      readonly modified: boolean;
      readonly screenshotUrl: string;
      readonly cloudScreenshotUrl: string;
      readonly localScreenshotUrl: string;
      readonly data: object;
      readonly timestamp: string;
      readonly storedLocally: boolean;
      readonly storedToCloud: boolean;
      readonly updated: boolean;
      readonly freezeFrames: Array<FreezeFrame>;
      clone(specification: ProjectSpecification): Project;
      loadState(): Promise<undefined>;
      saveState(screenshotUrl?: string): void;
      readFromCloud(): Promise<boolean>;
      removeLocally(): Promise<boolean>;
      storeLocally(): Promise<boolean>;
      storeToCloud(): Promise<boolean>;
      addFreezeFrame(): Promise<void>;
      downloadFreezeFrames(onProgress?: Function): Promise<Array<FreezeFrame>>;
      loadFreezeFrame(freezeFrame: FreezeFrame): Promise<void>;
      saveScreenshot(): Promise<string>;
      fetchData(): Promise<Object>;
    }

    type Projects = {
      local: Array<Project>;
      cloud: Array<Project>;
      all: Array<Project>;
    };

    interface ProjectManager {
      readonly projects: Projects;
      readonly online: boolean;
      usePublicCloudIndex: boolean;
      init(): Promise<undefined>;
      createNewProject(specification: ProjectSpecification): Project;
      updateCloudIndex(): Promise<boolean>;
      fetchProjectById(id: string): Promise<Project>;
    }

    interface Plugins {
      readonly list: Array<ViewarPlugin>;
      readonly instanceList: Array<object>;
      register(name: string, plugin: ViewarPlugin, params?: object): Promise<ViewarPlugin>;
      unregister(name: string): Promise<void>;
      get(name: string): ViewarPlugin;
      registerUiCallback(callbackFn: function): number;
      unregisterUiCallback(callbackId: number): void;
    }

    interface Profiler {
      start(updateInterval?: number): void;
      stop(): void;
    }

    interface FreezeFrame {
      name: string;
      thumbnailUrl: string;
      imageUrl: string;
    }

    interface Playable extends Emitter {
      start(time?: number, loop?: number): Promise<undefined>;
      stop(): Promise<undefined>;
      resume(): Promise<undefined>;
      pause(): Promise<undefined>;
    }

    interface Animation extends Playable {}

    interface Video extends Playable {
      name(): string;
      state(): string;
    }

    interface Option {
      readonly foreignKey: string;
      readonly foreignKeys: Array<string>;
      readonly imageUrl: string;
      readonly key: string;
      readonly name: string;
      readonly null: boolean;
      readonly tags: Array<string>;
      hasAllTags(query: Array<string>): boolean;
      hasAnyTag(query: Array<string>): boolean;
      hasExactTags(query: Array<string>): boolean;
    }

    interface Property {
      readonly aliases: Array<string>;
      readonly name: string;
      readonly type: string;
      readonly valueType: string;
      readonly value: Option;
      readonly options: Array<Option>;
      getInsertionParams(): object;
      getValueIterator(): IterableIterator<Option>;
      getValueKey(): string;
      hasValidValue: boolean;
    }

    interface SceneNode {
      readonly id: string;
      readonly pose: Pose;
      readonly interaction: Interaction;
      readonly visible: boolean;
      readonly parent: Container;
      readonly highlighted: boolean;
      readonly meta: MetaData;

      setPose(pose: Pose): Promise<undefined>;
      setInteraction(newInteraction: Interaction): Promise<undefined>;
      setVisible(visibility: boolean): Promise<undefined>;
      setParent(parent: Container): Promise<undefined>;
      setHighlighted(highlighted: boolean): Promise<undefined>;
      setMeta(metaChanges: object): object;
    }

    interface Model {
      readonly id: string;
      readonly name: string;
      readonly description: string;
      readonly imageUrl: string;
      readonly foreignKey: string;
      readonly tags: Array<string>;
      readonly data: object;
      readonly source: boolean;
      readonly shopUrl: string;
      readonly info: object;
      readonly type: string;
      readonly version: string;
      hasAllTags(query: Array<string>): boolean;
      hasAnyTag(query: Array<string>): boolean;
      hasExactTags(query: Array<string>): boolean;
      download(): Promise<undefined>;
      downloadDescription(): Promise<object>;
      update(): Promise<undefined>;
    }

    interface ModelInstance extends SceneNode {
      readonly model: Model;
      readonly allowExport: boolean;
      readonly animations: Array<Animation>;
      readonly displayTemplate: object;
      readonly insertionTime: number;
      readonly properties: StringTMap<Property>;
      readonly propertyValues: object;
      readonly videos: Array<Video>;
      readonly surfaceMaterials: object;
      setPropertyValues(newPropertyValues: object): Promise<undefined>;
      setSurfaceMaterials(newSurfaceMaterials: object): Promise<undefined>;
    }

    interface Container extends SceneNode {
      readonly type: string;
      readonly grouped: boolean;
      readonly children: Array<Node>;
    }

    interface Category {
      readonly id: string;
      readonly children: Array<Category>;
      readonly name: string;
      readonly imageUrl: string;
      readonly foreignKey: string;
      readonly tags: Array<string>;
      readonly data: object;
      hasAllTags(query: Array<string>): boolean;
      hasAnyTag(query: Array<string>): boolean;
      hasExactTags(query: Array<string>): boolean;
    }

    interface AuthenticationManager {
      readonly token: string;
      readonly userData: object;
      logIn(name: string, password?: string): Promise<boolean>;
      logOut(): void;
      setToken(token: string): void;
    }

    interface AppUtils {
      readonly microphoneMuted: boolean;
      readonly speakerMuted: boolean;
      closeApp(): Promise<undefined>;
      pauseRenderLoop(): Promise<undefined>;
      resumeRenderLoop(): Promise<undefined>;
      getAuthentication(): Promise<string>;
      setAuthentication(authKey: string): Promise<undefined>;
      httpPost(url: string, args: object): Promise<object>;
      sendEmail(emailObject: EmailObject): Promise<boolean>;
      openUrl(url: string): void;
      prepareCustomImage(fileName: string, url: string): string;
      setCollisionMode(collisionMode: number): Promise<undefined>;
      setTextureQuality(
        textureQuality: string,
        reloadScene?: boolean
      ): Promise<undefined>;
      getSecureUrl(url: string): string;
      getCachedUrl(url: string): string;
      getResourceInfo(display: boolean): Promise<object>;
      muteMicrophone(): Promise<undefined>;
      unmuteMicrophone(): Promise<undefined>;
      muteSpeaker(): Promise<undefined>;
      unmuteSpeaker(): Promise<undefined>;
    }

    interface ModelManager extends Emitter {
      readonly models: Array<Model>;
      readonly rootCategory: Category;
      findModelById(id: string): Model;
      findModelByForeignKey(foreignKey: string): Model;
      fetchModelFromRepository(id: string): Promise<Model>;
      fetchCategoryFromRepository(foreignKey: string): Promise<Category>;
      downloadAll(handleModelDownloadProgress?: Function): Promise<undefined>;
      stopDownloadAll(): Promise<undefined>;
    }

    interface RoomManager {
      readonly roomVisible: boolean;
      readonly roomMaterialDescription: object[];
      readonly roomDescription: RoomDescription;
      readonly wallObjectModels: Array<Model>;
      init(): Promise<undefined>;
      addRoomToScene(description: RoomDescription): Promise<undefined>;
      applyFloorMaterial(materialId: string): Promise<undefined>;
      getFloorSurface(): Promise<number>;
      getFloorPerimeter(): Promise<number>;
      removeRoomFromScene(): Promise<undefined>;
      setRoomVisible(visibility: boolean): Promise<undefined>;
      initResources(): Promise<undefined>;
    }

    interface SceneManager extends Emitter {
      readonly scene: Container;
      readonly selection: SceneNode;
      readonly selectionUpdation: boolean;
      readonly sceneTree: string;
      insertContainer(insertionParams: ContainerInsertionParams): Container;
      insertModel(
        model: Model,
        instanceProps?: ModelInsertionParams,
        insertionParams?: object
      ): ModelInstance;
      insertModels(
        bulkInsertionInfos: Array<BulkInsertionInfo>
      ): Array<ModelInstance>;
      removeNode(node: SceneNode): Promise<undefined>;
      select(node?: SceneNode): Promise<SceneNode>;
      clearSelection(): Promise<undefined>;
      findNodeById(id: string): SceneNode;
      clearScene(): Promise<undefined>;
      getSceneState(parent?: Container): object;
      getSceneStateSafe(parent?: Container): Promise<object>;
      setSceneState(sceneState: object, parent?: Container): Promise<undefined>;
      getRayIntersections(
        origin: Vector3,
        direction: Vector3,
        featurePointRadius: number
      ): Promise<RayHits>;
      simulateTouchRay(
        screenX: number,
        screenY: number,
        featurePointRadius: number
      ): Promise<RayHits>;
      exportScene(
        sceneName: string,
        format: string
      ): Promise<SceneExportResult>;
      startSelectionUpdate(): Promise<undefined>;
      stopSelectionUpdate(): Promise<undefined>;
    }

    interface CoreInterface extends Emitter {
      readonly platform: string;
      readonly history: Array<object>;
      call(callName: string, ...messageArguments: Array<any>);
      emit(eventName: string, ...messageArguments: Array<any>);
      resolveUrl(relativeUrl: string): string;
    }

    interface Camera {
      readonly active: boolean;
      readonly name: string;
      readonly poseUpdating: string;
      activate(): Promise<undefined>;
      enableHmdMode(): Promise<undefined>;
      disableHmdMode(): Promise<undefined>;
      getPoseInViewingDirection(
        distance: number,
        projectToFloor?: boolean
      ): Promise<Pose>;
      getColorsAtWorldPositions(
        positions: Array<Vector3>,
        radius?: number
      ): Promise<Array<ColorRgb>>;
      updatePose(): Promise<Pose>;
      startPoseUpdate(): Promise<undefined>;
      stopPoseUpdate(): Promise<undefined>;
    }

    interface ARCamera extends Camera {
      readonly pose: ControllableCameraPose;
      readonly activeFreezeFrame: FreezeFrame;
      readonly frozen: boolean;
      readonly freezeFrames: Array<FreezeFrame>;
      readonly pointCloudVisible: boolean;
      freeze(): Promise<undefined>;
      unfreeze(): Promise<undefined>;
      showFreezeFrame(freezeFrame: FreezeFrame): Promise<undefined>;
      removeFreezeFrame(freezeFrame: FreezeFrame): Promise<undefined>;
      saveFreezeFrame(name: string): Promise<FreezeFrame>;
      downloadFreezeFrame(name: string): Promise<FreezeFrame>;
      showPointCloud(): Promise<undefined>;
      hidePointCloud(): Promise<undefined>;
    }

    interface PerspectiveCamera extends Camera {
      readonly pose: PerspectiveCameraPose;
      readonly interaction: Interaction;
      disableSceneInteraction(): Promise<undefined>;
      enableSceneInteraction(): Promise<undefined>;
      setInteraction(newInteraction: Interaction): Promise<undefined>;
      setPose(newPose: PerspectiveCameraPose): Promise<undefined>;
      setBackgroundMaterial(materialId: string): Promise<undefined>;
      zoomToFit(): Promise<undefined>;
      lookAtSceneCenter(): Promise<undefined>;
    }

    interface VRCamera extends Camera {
      readonly pose: ControllableCameraPose;
      readonly gyroscopeActive: boolean;
      enableGyroscope(): Promise<undefined>;
      disableGyroscope(): Promise<undefined>;
      translate(controllerInputVector: Vector3): void;
      rotate(controllerInputVector: Vector3): void;
    }

    interface Cameras {
      readonly activeCamera: Camera;
      readonly arCamera: ARCamera;
      readonly perspectiveCamera: PerspectiveCamera;
      readonly vrCamera: VRCamera;
    }

    interface Http {
      get(url: string, data: object, timeout?: number): Promise<object>;
      post(url: string, data: object, timeout?: number): Promise<object>;
    }

    interface LogOutput {
      error(...messages: Array<string>): void;
      log(...messages: Array<string>): void;
      warn(...messages: Array<string>): void;
      info(...messages: Array<string>): void;
    }

    interface Logger extends LogOutput {
      register(output: Logger);
    }

    declare class ViewarPlugin {
      getType(): string;
      initialize(): Promise<void>;
      destroy(): Promise<void>;
    }

    declare class ViewarModelPlugin extends ViewarPlugin {
      getModels(): Promise<Array<Model>>;
      getSceneState(model: Model, pose: Pose): Promise<object>;
      postProcess(container: Container): Promise<void>;
      getProperties(model: Model): StringTMap<Property>;
      getPropertyValues(model: Model): object;
      setPropertyValues(propertyValues: object, model: Model): Promise<void>;
    }

    //-------------
    // Singletons
    //-------------
    // Export as constant and define interface. With modelManager as namespace we can't inherit from emitter.

    export const ViewarModelPlugin;
    export const ViewarPlugin;
    export const appConfig: AppConfig;
    export const appUtils: AppUtils;
    export const authenticationManager: AuthenticationManager;
    export const cameras: Cameras;
    export const coreInterface: CoreInterface;
    export const http: Http;
    export const logger: Logger;
    export const modelManager: ModelManager;
    export const plugins: Plugins;
    export const profiler: Profiler;
    export const projectManager: ProjectManager;
    export const roomManager: RoomManager;
    export const sceneManager: SceneManager;
    export const screenshotManager: ScreenshotManager;
    export const storage: Storages;
    export const syncManager: SyncManager;
    export const tracker: Tracker;
    export const trackers: Trackers;
    export const versionInfo: VersionInfo;
  }

  export = viewarApi;
}
